/*
   +----------------------------------------------------------------------+
   | PHP Version 7                                                        |
   +----------------------------------------------------------------------+
   | Copyright (c) The PHP Group                                          |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
   | available through the world-wide-web at the following url:           |
   | http://www.php.net/license/3_01.txt                                  |
   | If you did not receive a copy of the PHP license and are unable to   |
   | obtain it through the world-wide-web, please send a note to          |
   | license@php.net so we can mail you a copy immediately.               |
   +----------------------------------------------------------------------+
   | Author: Stig Sæther Bakken <ssb@php.net>                             |
   +----------------------------------------------------------------------+
*/

#define CONFIGURE_COMMAND " './configure'  '--disable-dependency-tracking' '--prefix=/usr' '--exec-prefix=/usr' '--bindir=/usr/bin' '--sbindir=/usr/sbin' '--sysconfdir=/etc' '--datadir=/usr/share' '--includedir=/usr/include' '--libdir=/usr/lib64' '--libexecdir=/usr/libexec' '--sharedstatedir=/var/lib' '--mandir=/usr/share/man' '--infodir=/usr/share/info' '--build=x86_64-redhat-linux-gnu' '--host=x86_64-redhat-linux-gnu' '--target=x86_64-redhat-linux-gnu' '--program-prefix=' '--prefix=/opt/alt/php74' '--exec-prefix=/opt/alt/php74' '--bindir=/opt/alt/php74/usr/bin' '--sbindir=/opt/alt/php74/usr/sbin' '--sysconfdir=/opt/alt/php74/etc' '--datadir=/opt/alt/php74/usr/share' '--includedir=/opt/alt/php74/usr/include' '--libdir=/opt/alt/php74/usr/lib64' '--libexecdir=/opt/alt/php74/usr/libexec' '--localstatedir=/var' '--sharedstatedir=/usr/com' '--mandir=/opt/alt/php74/usr/share/man' '--infodir=/opt/alt/php74/usr/share/info' '--cache-file=../config.cache' '--with-libdir=lib64' '--with-config-file-path=/opt/alt/php74/etc' '--with-config-file-scan-dir=/opt/alt/php74/link/conf' '--disable-debug' '--enable-calendar' '--enable-exif' '--enable-ftp' '--enable-huge-code-pages' '--enable-jit' '--enable-shmop' '--enable-xml' '--with-bz2' '--with-freetype=/usr' '--with-gettext' '--with-gmp' '--with-iconv' '--with-jpeg=/usr' '--with-layout=GNU' '--with-mhash' '--with-password-argon2=/usr' '--with-pcre-jit' '--with-pic' '--with-readline' '--with-webp=/opt/alt/libwebp' '--with-xpm=/usr' '--with-zlib' '--with-zlib-dir=/usr' '--without-gdbm' '--without-pear' '--with-curl=/opt/alt/curlssl11/usr' '--with-openssl-dir=/opt/alt/openssl11' '--with-openssl=/opt/alt/openssl11' '--with-pcre-dir=/opt/alt/pcre2' '--with-kerberos=/opt/alt/krb5/usr' '--with-apxs2' '--enable-litespeed' '--enable-fpm' '--with-fpm-systemd' '--enable-maintainer-zts=no' '--enable-bcmath=shared' '--enable-dba=shared' '--with-db4=/usr' '--enable-dom=shared' '--enable-fileinfo=shared' '--enable-gd=shared' '--enable-intl=shared' '--enable-json=shared' '--enable-mbregex' '--enable-mbstring=shared' '--enable-mysqlnd=shared' '--enable-opcache' '--enable-pcntl' '--enable-pdo=shared' '--enable-phar=shared' '--enable-posix=shared' '--enable-soap=shared' '--enable-sockets=shared' '--enable-sqlite3=/opt/alt/sqlite/usr' '--enable-sysvsem=shared' '--enable-sysvshm=shared' '--enable-sysvmsg=shared' '--enable-xmlreader=shared' '--enable-xmlwriter=shared' '--with-enchant=shared,/usr' '--with-external-gd' '--with-mysql-sock=/var/lib/mysql/mysql.sock' '--with-mysqli=shared,mysqlnd' '--with-pdo-mysql=shared,mysqlnd' '--with-pdo-odbc=shared,unixODBC,/usr' '--with-pdo-sqlite=shared,/opt/alt/sqlite/usr' '--with-pspell=shared' '--with-tidy=shared,/usr' '--with-unixODBC=shared,/usr' '--with-xmlrpc=shared' '--with-zip=shared' '--with-pdo-pgsql=shared,/usr' '--with-pgsql=shared,/usr' '--with-imap=shared,/opt/alt/libc-client11' '--with-imap-ssl=/opt/alt/openssl11' '--with-ldap=shared,/opt/alt/openldap11' '--with-ldap-sasl' '--with-snmp=shared,/opt/alt/net-snmp11/usr' '--with-pdo-dblib=shared,/opt/alt/freetds11/usr' '--with-pdo-oci=shared,instantclient,/usr/lib/oracle/21/client64/lib' '--with-pdo-firebird=shared' '--with-sodium=shared,/usr' '--with-xsl=shared,/usr' 'build_alias=x86_64-redhat-linux-gnu' 'host_alias=x86_64-redhat-linux-gnu' 'target_alias=x86_64-redhat-linux-gnu' 'PKG_CONFIG=/usr/bin/pkg-config' 'PKG_CONFIG_PATH=/opt/alt/openssl11/lib64/pkgconfig:/opt/alt/pcre2/usr/lib64/pkgconfig:/opt/alt/libzip/usr/lib64/pkgconfig:/opt/alt/krb5/usr/lib64/pkgconfig:/opt/alt/libxml2/usr/lib64/pkgconfig:/opt/alt/libicu71/usr/lib64/pkgconfig:/opt/alt/libgd/lib64/pkgconfig:/opt/alt/curlssl11/usr/lib64/pkgconfig:/opt/alt/sqlite/usr/lib64/pkgconfig' 'CFLAGS=-O2 -g -pipe -Wall -Werror=format-security -Wp,-D_FORTIFY_SOURCE=2 -Wp,-D_GLIBCXX_ASSERTIONS -fexceptions -fstack-protector-strong -grecord-gcc-switches -specs=/usr/lib/rpm/redhat/redhat-hardened-cc1 -specs=/usr/lib/rpm/redhat/redhat-annobin-cc1 -m64 -mtune=generic -fasynchronous-unwind-tables -fstack-clash-protection -fcf-protection -fno-strict-aliasing -Wno-pointer-sign -mshstk -I/opt/alt/t1lib/usr/include -I/opt/alt/libc-client11/include -I/opt/alt/libssh211/usr/include' 'KERBEROS_CFLAGS=-I/opt/alt/krb5/usr/include' 'KERBEROS_LIBS=-L/opt/alt/krb5/usr/lib64' 'WEBP_LIBS=-L/opt/alt/libwebp/lib64 -Wl,-rpath=/opt/alt/libwebp/lib64' 'SASL_CFLAGS=-I/opt/alt/cyrus-sasl/include' 'SASL_LIBS=-L/opt/alt/cyrus-sasl/lib64'"
#define PHP_ODBC_CFLAGS	"-I/usr/include"
#define PHP_ODBC_LFLAGS		"-L/usr/lib64"
#define PHP_ODBC_LIBS		"-lodbc"
#define PHP_ODBC_TYPE		"unixODBC"
#define PHP_OCI8_DIR			""
#define PHP_OCI8_ORACLE_VERSION		""
#define PHP_PROG_SENDMAIL	"/usr/sbin/sendmail"
#define PEAR_INSTALLDIR         "/opt/alt/php74/usr/share/pear"
#define PHP_INCLUDE_PATH	".:/opt/alt/php74/usr/share/pear"
#define PHP_EXTENSION_DIR       "/opt/alt/php74/usr/lib64/php/modules"
#define PHP_PREFIX              "/opt/alt/php74"
#define PHP_BINDIR              "/opt/alt/php74/usr/bin"
#define PHP_SBINDIR             "/opt/alt/php74/usr/sbin"
#define PHP_MANDIR              "/opt/alt/php74/usr/share/man"
#define PHP_LIBDIR              "/opt/alt/php74/usr/lib64"
#define PHP_DATADIR             "/opt/alt/php74/usr/share"
#define PHP_SYSCONFDIR          "/opt/alt/php74/etc"
#define PHP_LOCALSTATEDIR       "/var"
#define PHP_CONFIG_FILE_PATH    "/opt/alt/php74/etc"
#define PHP_CONFIG_FILE_SCAN_DIR    "/opt/alt/php74/link/conf"
#define PHP_SHLIB_SUFFIX        "so"
#define PHP_SHLIB_EXT_PREFIX    ""
